package com.notificationFramework.notification;

import android.annotation.TargetApi;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.app.TaskStackBuilder;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Build;
import android.support.v4.app.NotificationCompat;

import com.notificationFramework.sedentary.frontEnd.NotificationClicked;
import com.notificationFramework.sedentary.frontEnd.R;
import com.notificationFramework.sedentary.frontEnd.SaveFile;

/**
 * Created by Peter De Jonckheere on 09/01/2018.
 * <p>
 * The abstract Service which provides the basis for creating and sending a notification. This
 * class delegates to sub-classes for the construction of the notification and its related channel
 * in terms of properties which are unique to each notification such as content and content title.
 * This class is to be extended to create a new variant on the notification with the 2 abstract
 * methods being implemented.
 * </p>
 */

public abstract class Notification extends Service {

    /**
     * The instance of NotificationManager.class which is used to send the notification
     */
    protected NotificationManager nm;
    /**
     * Arbitrary integer for the id of the notification
     */
    protected int snId = 0;
    /**
     * Instance of SharedPreferences which allows access to information saved in
     * sedentary.FrontEnd.Settings needed to determine the properties of the notification to
     * be sent.
     */
    protected SharedPreferences preferences;

    /**
     * Called when the service to send a notification is started from another place in the program.
     * Sets up the NotificationManager and SharedPreferences then delegates to buildNotification()
     * method to create and send the notification.
     *
     * @param intent  the intent used to start the service
     * @param flags   additional information about the start request
     * @param startId the unique start id of this service
     * @return defines the start conditions for the OS when starting this service
     * @see android.app.Service
     */
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        nm = (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);
        preferences = this.getSharedPreferences(getString(R.string.preference_file_key), Context.MODE_PRIVATE);
        buildNotification();
        return START_STICKY;
    }

    /**
     * Sets up the notification structure, delegating to setParameters() and setChannelParameters()
     * in concrete implementations for individual properties of notifications. Differentiates
     * between API to determine if a NotificationChannel is required also constrcuts the
     * stack and intent which allows an intent to be fired upon clicking on the notification.
     * The notification is then sent to the NotificationManager using notify() and the notification
     * service stops itself.
     */
    protected void buildNotification() {
        NotificationCompat.Builder mBuilder;
        //To find if a notification channel is required
        if (Build.VERSION.SDK_INT >= 26) {
            NotificationChannel nc = createNotificationChannel();
            mBuilder = new NotificationCompat.Builder(this, nc.getId());
        } else {
            mBuilder = new NotificationCompat.Builder(this, "sednc");
        }
        mBuilder = setParameters(mBuilder);
        //Creates and attaches the intent to the notification which links the notification click
        //to an activity
        Intent resultIntent = new Intent(this, NotificationClicked.class);
        TaskStackBuilder stackBuilder = TaskStackBuilder.create(this);
        stackBuilder.addParentStack(NotificationClicked.class);
        stackBuilder.addNextIntent(resultIntent);
        PendingIntent resultPendingIntent =
                stackBuilder.getPendingIntent(
                        0,
                        PendingIntent.FLAG_UPDATE_CURRENT
                );
        mBuilder.setContentIntent(resultPendingIntent);
        nm.notify(snId, mBuilder.build());
        //To record information about the notification sent
        SaveFile.recordNotification(1, 0, 0, this);
        SaveFile.recordNotificationType(this);
        //Stops this service
        stopSelf();
    }

    /**
     * Creates a new NotificationChannel.
     *
     * @return the created NotificationChannel
     */
    @TargetApi(26)
    private NotificationChannel createNotificationChannel() {
        NotificationChannel nc = new NotificationChannel("sednc", "sedentaryNotification",
                NotificationManager.IMPORTANCE_DEFAULT);
        nc = setChannelParameters(nc);
        return nc;
    }

    /**
     * To be implemented by concrete implementation of this class and used to define the unique
     * properties of the notification such as the content title and content text.
     *
     * @param builder the NotificationCompat.Builder provided to be altered with unique properties
     * @return the altered version of 'builder'
     */
    protected abstract NotificationCompat.Builder setParameters(NotificationCompat.Builder builder);

    /**
     * To be implemented by concrete implementations of this class and used to define the unique
     * properties of the NotificationChannel when required
     *
     * @param nc the NotificationChannel to be altered with individual properties
     * @return the altered version of 'nc'
     */
    protected abstract NotificationChannel setChannelParameters(NotificationChannel nc);
}
